// -*- C++ -*-

/* 
 * GChemPaint
 * molecule.cc 
 *
 * Copyright (C) 2001-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include "molecule.h"
#include "document.h"

gcpMolecule::gcpMolecule(TypeId Type): Object(Type)
{
}

gcpMolecule::gcpMolecule(gcpAtom* pAtom): Object(MoleculeType)
{
	AddAtom(pAtom);
	gcpChain* pChain = new gcpChain(this, pAtom); //will find the cycles
	delete pChain;
}

gcpMolecule::~gcpMolecule()
{
	std::list<gcpAtom*>::iterator i;
	std::list<gcpBond*>::iterator n;
	for (n = m_Bonds.begin(); n != m_Bonds.end(); n++) (*n)->RemoveAllCycles();
	while (!m_Cycles.empty())
	{
		delete m_Cycles.front();
		m_Cycles.pop_front();
	}
	while (!m_Chains.empty())
	{
		delete m_Chains.front();
		m_Chains.pop_front();
	}
}

void gcpMolecule::AddAtom(gcpAtom* pAtom)
{
	m_Atoms.push_back(pAtom);
	AddChild(pAtom);
}

void gcpMolecule::AddBond(gcpBond* pBond)
{
	m_Bonds.push_back(pBond);
	AddChild(pBond);
}

void gcpMolecule::Remove(gcpBond* pBond)
{
	m_Bonds.remove(pBond);
	pBond->SetParent(GetParent());
}

void gcpMolecule::UpdateCycles(gcpBond* pBond)	//FIXME: function not totally implemented
{
	gcpChain* pChain = new gcpChain(this, pBond); //will find the cycles
	delete pChain;
}

void gcpMolecule::Merge(gcpMolecule* pMolecule)
{
	gcpAtom* pAtom;
	gcpBond* pBond;
	gcpChain* pChain;
	gcpCycle* pCycle;
	pAtom = m_Atoms.front();
	while (!pMolecule->m_Atoms.empty())
	{
		pAtom = pMolecule->m_Atoms.front();
		AddAtom(pAtom);
		pMolecule->m_Atoms.pop_front();
	}
	while (!pMolecule->m_Bonds.empty())
	{
		pBond = pMolecule->m_Bonds.front();
		AddBond(pBond);
		pMolecule->m_Bonds.pop_front();
	}
	while (!pMolecule->m_Chains.empty())	//FIXME: Chains should change
	{
		pChain = pMolecule->m_Chains.front();
		m_Chains.push_back(pChain);
		pMolecule->m_Chains.pop_front();
	}
	while (!pMolecule->m_Cycles.empty())
	{
		pCycle = pMolecule->m_Cycles.front();
		m_Cycles.push_back(pCycle);
		pMolecule->m_Cycles.pop_front();
	}
	delete pMolecule;
}

xmlNodePtr gcpMolecule::Save(xmlDocPtr xml)
{
	xmlNodePtr node;
	node = xmlNewDocNode(xml, NULL, (xmlChar*)"molecule", NULL);
	if (!node) return NULL;
	xmlNewProp(node, (xmlChar*)"id", (xmlChar*)m_Id);
	
	if (!SaveChildren(xml, node))
	{
		xmlFreeNode(node);
		return NULL;
	}
	return node;
}

bool gcpMolecule::Load(xmlNodePtr node)
{
	char* tmp;
	xmlNodePtr child;
	Object* pObject;
	gcpDocument* pDoc = (gcpDocument*)GetDocument();

	tmp = (char*)xmlGetProp(node, (xmlChar*)"id");
	if (tmp) SetId(tmp);
	child = GetNodeByName(node, "atom");
	while (child)
	{
		pObject = new gcpAtom();
		if (!pObject->Load(child)) 
		{
			delete pObject;
			return false;
		}
		if (pDoc) pDoc->AddAtom((gcpAtom*)pObject);
		AddAtom((gcpAtom*)pObject);
		child = GetNextNodeByName(child->next, "atom");
	}

	child = GetNodeByName(node, "bond");
	while (child)
	{
		pObject = new gcpBond();
		AddBond((gcpBond*)pObject);
		if (!pObject->Load(child))
		{
			delete pObject;
			m_Bonds.remove((gcpBond*)pObject);
			return false;
		}
		if (pDoc) pDoc->AddBond((gcpBond*)pObject);
		child = GetNextNodeByName(child->next, "bond");
	}
	gcpAtom* pAtom = m_Atoms.front();
	std::list<gcpAtom*>::iterator i = m_Atoms.begin();
	i++;
	for (; i != m_Atoms.end(); i++) (*i)->SetParent(NULL);
	gcpChain* pChain = new gcpChain(this, pAtom); //will find the cycles
	delete pChain;
	return true;
}

void gcpMolecule::UpdateCycles()
{
	std::list<gcpBond*>::iterator n;
	for (n = m_Bonds.begin(); n != m_Bonds.end(); n++) (*n)->RemoveAllCycles();
	while (!m_Cycles.empty())
	{
		delete m_Cycles.front();
		m_Cycles.pop_front();
	}
	std::list<gcpAtom*>::iterator i = m_Atoms.begin();
	i++;
	for (; i != m_Atoms.end(); i++) (*i)->SetParent(NULL);
	gcpChain* pChain = new gcpChain(this, *(m_Atoms.begin())); //will find the cycles
	delete pChain;
}

void gcpMolecule::Clear()
{
	m_Bonds.clear();
	m_Atoms.clear();
}
